@@ -18,17 +18,10 @@ from __future__ import division
 from __future__ import print_function
 
 import os
-import sys
 import tempfile
 import time
 
-# pylint: disable=g-import-not-at-top
-
-# TODO(jart): #6568 Remove this hack that makes dlopen() not crash.
-if hasattr(sys, "getdlopenflags") and hasattr(sys, "setdlopenflags"):
-  import ctypes
-  sys.setdlopenflags(sys.getdlopenflags() | ctypes.RTLD_GLOBAL)
-
+from tensorflow.contrib.layers.python.layers import feature_column as fc
 from tensorflow.contrib.learn.python.learn import export_strategy as export_strategy_lib
 from tensorflow.contrib.learn.python.learn.estimators import constants
 from tensorflow.contrib.learn.python.learn.estimators import model_fn
@@ -44,6 +37,7 @@ from tensorflow.python.platform import gfile
 from tensorflow.python.platform import test
 from tensorflow.python.saved_model import signature_constants
 from tensorflow.python.saved_model import signature_def_utils
+from tensorflow.python.util import compat
 
 
 class SavedModelExportUtilsTest(test.TestCase):
@@ -244,6 +238,95 @@ class SavedModelExportUtilsTest(test.TestCase):
         signature_constants.CLASSIFY_METHOD_NAME)
     self.assertEqual(actual_signature_def, expected_signature_def)
 
+  def test_build_standardized_signature_def_classification5(self):
+    """Tests multiple output tensors that include integer classes and scores.
+
+    Integer classes are dropped out, because Servo classification can only serve
+    string classes. So, only scores are present in the signature.
+    """
+    input_tensors = {
+        "input-1":
+            array_ops.placeholder(
+                dtypes.float32, 1, name="input-tensor-1")
+    }
+    output_tensors = {
+        "classes":
+            array_ops.placeholder(
+                dtypes.int64, 1, name="output-tensor-classes"),
+        "scores":
+            array_ops.placeholder(
+                dtypes.float32, 1, name="output-tensor-scores"),
+        "logits":
+            array_ops.placeholder(
+                dtypes.float32, 1, name="output-tensor-logits-unused"),
+    }
+    problem_type = constants.ProblemType.CLASSIFICATION
+    actual_signature_def = (
+        saved_model_export_utils.build_standardized_signature_def(
+            input_tensors, output_tensors, problem_type))
+    expected_signature_def = meta_graph_pb2.SignatureDef()
+    shape = tensor_shape_pb2.TensorShapeProto(
+        dim=[tensor_shape_pb2.TensorShapeProto.Dim(size=1)])
+    dtype_float = types_pb2.DataType.Value("DT_FLOAT")
+    expected_signature_def.inputs[
+        signature_constants.CLASSIFY_INPUTS].CopyFrom(
+            meta_graph_pb2.TensorInfo(
+                name="input-tensor-1:0", dtype=dtype_float, tensor_shape=shape))
+    expected_signature_def.outputs[
+        signature_constants.CLASSIFY_OUTPUT_SCORES].CopyFrom(
+            meta_graph_pb2.TensorInfo(
+                name="output-tensor-scores:0", dtype=dtype_float,
+                tensor_shape=shape))
+
+    expected_signature_def.method_name = (
+        signature_constants.CLASSIFY_METHOD_NAME)
+    self.assertEqual(actual_signature_def, expected_signature_def)
+
+  def test_build_standardized_signature_def_classification6(self):
+    """Tests multiple output tensors that with integer classes and no scores.
+
+    Servo classification cannot serve integer classes, but no scores are
+    available. So, we fall back to predict signature.
+    """
+    input_tensors = {
+        "input-1":
+            array_ops.placeholder(
+                dtypes.float32, 1, name="input-tensor-1")
+    }
+    output_tensors = {
+        "classes":
+            array_ops.placeholder(
+                dtypes.int64, 1, name="output-tensor-classes"),
+        "logits":
+            array_ops.placeholder(
+                dtypes.float32, 1, name="output-tensor-logits"),
+    }
+    problem_type = constants.ProblemType.CLASSIFICATION
+    actual_signature_def = (
+        saved_model_export_utils.build_standardized_signature_def(
+            input_tensors, output_tensors, problem_type))
+    expected_signature_def = meta_graph_pb2.SignatureDef()
+    shape = tensor_shape_pb2.TensorShapeProto(
+        dim=[tensor_shape_pb2.TensorShapeProto.Dim(size=1)])
+    dtype_int64 = types_pb2.DataType.Value("DT_INT64")
+    dtype_float = types_pb2.DataType.Value("DT_FLOAT")
+    expected_signature_def.inputs[
+        signature_constants.PREDICT_INPUTS].CopyFrom(
+            meta_graph_pb2.TensorInfo(
+                name="input-tensor-1:0", dtype=dtype_float, tensor_shape=shape))
+    expected_signature_def.outputs["classes"].CopyFrom(
+        meta_graph_pb2.TensorInfo(
+            name="output-tensor-classes:0", dtype=dtype_int64,
+            tensor_shape=shape))
+    expected_signature_def.outputs["logits"].CopyFrom(
+        meta_graph_pb2.TensorInfo(
+            name="output-tensor-logits:0", dtype=dtype_float,
+            tensor_shape=shape))
+
+    expected_signature_def.method_name = (
+        signature_constants.PREDICT_METHOD_NAME)
+    self.assertEqual(actual_signature_def, expected_signature_def)
+
   def test_get_input_alternatives(self):
     input_ops = input_fn_utils.InputFnOps("bogus features dict", None,
                                           "bogus default input dict")
@@ -257,7 +340,7 @@ class SavedModelExportUtilsTest(test.TestCase):
     #     saved_model_export_utils.FEATURES_INPUT_ALTERNATIVE_KEY],
     #                  "bogus features dict")
 
-  def test_get_output_alternatives_explicit(self):
+  def test_get_output_alternatives_explicit_default(self):
     provided_output_alternatives = {
         "head-1": (constants.ProblemType.LINEAR_REGRESSION,
                    "bogus output dict"),
@@ -268,12 +351,70 @@ class SavedModelExportUtilsTest(test.TestCase):
         model_fn.ModeKeys.INFER,
         predictions={"some_output": "bogus_tensor"},
         output_alternatives=provided_output_alternatives)
+
     output_alternatives, _ = saved_model_export_utils.get_output_alternatives(
         model_fn_ops, "head-1")
 
     self.assertEqual(provided_output_alternatives, output_alternatives)
 
-  def test_get_output_alternatives_implicit(self):
+  def test_get_output_alternatives_wrong_default(self):
+    provided_output_alternatives = {
+        "head-1": (constants.ProblemType.LINEAR_REGRESSION,
+                   "bogus output dict"),
+        "head-2": (constants.ProblemType.CLASSIFICATION, "bogus output dict 2"),
+        "head-3": (constants.ProblemType.UNSPECIFIED, "bogus output dict 3"),
+    }
+    model_fn_ops = model_fn.ModelFnOps(
+        model_fn.ModeKeys.INFER,
+        predictions={"some_output": "bogus_tensor"},
+        output_alternatives=provided_output_alternatives)
+
+    with self.assertRaises(ValueError) as e:
+      saved_model_export_utils.get_output_alternatives(model_fn_ops, "WRONG")
+
+    self.assertEqual("Requested default_output_alternative: WRONG, but "
+                     "available output_alternatives are: ['head-1', 'head-2', "
+                     "'head-3']", str(e.exception))
+
+  def test_get_output_alternatives_single_no_default(self):
+    prediction_tensor = constant_op.constant(["bogus"])
+    provided_output_alternatives = {
+        "head-1": (constants.ProblemType.LINEAR_REGRESSION,
+                   {"output": prediction_tensor}),
+    }
+    model_fn_ops = model_fn.ModelFnOps(
+        model_fn.ModeKeys.INFER,
+        predictions=prediction_tensor,
+        output_alternatives=provided_output_alternatives)
+
+    output_alternatives, _ = saved_model_export_utils.get_output_alternatives(
+        model_fn_ops)
+
+    self.assertEqual({"head-1":
+                      (constants.ProblemType.LINEAR_REGRESSION,
+                       {"output": prediction_tensor})},
+                     output_alternatives)
+
+  def test_get_output_alternatives_multi_no_default(self):
+    provided_output_alternatives = {
+        "head-1": (constants.ProblemType.LINEAR_REGRESSION,
+                   "bogus output dict"),
+        "head-2": (constants.ProblemType.CLASSIFICATION, "bogus output dict 2"),
+        "head-3": (constants.ProblemType.UNSPECIFIED, "bogus output dict 3"),
+    }
+    model_fn_ops = model_fn.ModelFnOps(
+        model_fn.ModeKeys.INFER,
+        predictions={"some_output": "bogus_tensor"},
+        output_alternatives=provided_output_alternatives)
+
+    with self.assertRaises(ValueError) as e:
+      saved_model_export_utils.get_output_alternatives(model_fn_ops)
+
+    self.assertEqual("Please specify a default_output_alternative.  Available "
+                     "output_alternatives are: ['head-1', 'head-2', 'head-3']",
+                     str(e.exception))
+
+  def test_get_output_alternatives_none_provided(self):
     prediction_tensor = constant_op.constant(["bogus"])
     model_fn_ops = model_fn.ModelFnOps(
         model_fn.ModeKeys.INFER,
@@ -281,12 +422,40 @@ class SavedModelExportUtilsTest(test.TestCase):
         output_alternatives=None)
 
     output_alternatives, _ = saved_model_export_utils.get_output_alternatives(
-        model_fn_ops, "some_output")
-    self.assertEqual({
-        "default_output_alternative": (constants.ProblemType.UNSPECIFIED, {
-            "some_output": prediction_tensor
-        })
-    }, output_alternatives)
+        model_fn_ops)
+
+    self.assertEqual(
+        {"default_output_alternative": (constants.ProblemType.UNSPECIFIED, {
+            "some_output": prediction_tensor})},
+        output_alternatives)
+
+  def test_get_output_alternatives_empty_provided_with_default(self):
+    prediction_tensor = constant_op.constant(["bogus"])
+    model_fn_ops = model_fn.ModelFnOps(
+        model_fn.ModeKeys.INFER,
+        predictions={"some_output": prediction_tensor},
+        output_alternatives={})
+
+    with self.assertRaises(ValueError) as e:
+      saved_model_export_utils.get_output_alternatives(model_fn_ops, "WRONG")
+
+    self.assertEqual("Requested default_output_alternative: WRONG, but "
+                     "available output_alternatives are: []", str(e.exception))
+
+  def test_get_output_alternatives_empty_provided_no_default(self):
+    prediction_tensor = constant_op.constant(["bogus"])
+    model_fn_ops = model_fn.ModelFnOps(
+        model_fn.ModeKeys.INFER,
+        predictions={"some_output": prediction_tensor},
+        output_alternatives={})
+
+    output_alternatives, _ = saved_model_export_utils.get_output_alternatives(
+        model_fn_ops)
+
+    self.assertEqual(
+        {"default_output_alternative": (constants.ProblemType.UNSPECIFIED, {
+            "some_output": prediction_tensor})},
+        output_alternatives)
 
   def test_get_output_alternatives_implicit_single(self):
     prediction_tensor = constant_op.constant(["bogus"])
@@ -403,21 +572,21 @@ class SavedModelExportUtilsTest(test.TestCase):
     export_dir_base = tempfile.mkdtemp() + "export/"
     export_dir_1 = saved_model_export_utils.get_timestamped_export_dir(
         export_dir_base)
-    time.sleep(0.001)
+    time.sleep(2)
     export_dir_2 = saved_model_export_utils.get_timestamped_export_dir(
         export_dir_base)
-    time.sleep(0.001)
+    time.sleep(2)
     export_dir_3 = saved_model_export_utils.get_timestamped_export_dir(
         export_dir_base)
 
-    # Export directories should be named using a timestamp that is milliseconds
-    # since epoch.  Such a timestamp is 13 digits long.
+    # Export directories should be named using a timestamp that is seconds
+    # since epoch.  Such a timestamp is 10 digits long.
     time_1 = os.path.basename(export_dir_1)
-    self.assertEqual(13, len(time_1))
+    self.assertEqual(10, len(time_1))
     time_2 = os.path.basename(export_dir_2)
-    self.assertEqual(13, len(time_2))
+    self.assertEqual(10, len(time_2))
     time_3 = os.path.basename(export_dir_3)
-    self.assertEqual(13, len(time_3))
+    self.assertEqual(10, len(time_3))
 
     self.assertTrue(int(time_1) < int(time_2))
     self.assertTrue(int(time_2) < int(time_3))
@@ -444,12 +613,30 @@ class SavedModelExportUtilsTest(test.TestCase):
     self.assertTrue(gfile.Exists(export_dir_3))
     self.assertTrue(gfile.Exists(export_dir_4))
 
+  def test_get_most_recent_export(self):
+    export_dir_base = tempfile.mkdtemp() + "export/"
+    gfile.MkDir(export_dir_base)
+    _create_test_export_dir(export_dir_base)
+    _create_test_export_dir(export_dir_base)
+    _create_test_export_dir(export_dir_base)
+    export_dir_4 = _create_test_export_dir(export_dir_base)
+
+    (most_recent_export_dir, most_recent_export_version) = (
+        saved_model_export_utils.get_most_recent_export(export_dir_base))
+
+    self.assertEqual(compat.as_bytes(export_dir_4),
+                     compat.as_bytes(most_recent_export_dir))
+    self.assertEqual(compat.as_bytes(export_dir_4),
+                     os.path.join(compat.as_bytes(export_dir_base),
+                                  compat.as_bytes(
+                                      str(most_recent_export_version))))
+
   def test_make_export_strategy(self):
     """Only tests that an ExportStrategy instance is created."""
-    def _export_input_fn():
+    def _serving_input_fn():
       return array_ops.constant([1]), None
     export_strategy = saved_model_export_utils.make_export_strategy(
-        export_input_fn=_export_input_fn,
+        serving_input_fn=_serving_input_fn,
         default_output_alternative_key="default",
         assets_extra={"from/path": "to/path"},
         as_text=False,
@@ -457,12 +644,31 @@ class SavedModelExportUtilsTest(test.TestCase):
     self.assertTrue(
         isinstance(export_strategy, export_strategy_lib.ExportStrategy))
 
+  def test_make_parsing_export_strategy(self):
+    """Only tests that an ExportStrategy instance is created."""
+    sparse_col = fc.sparse_column_with_hash_bucket(
+        "sparse_column", hash_bucket_size=100)
+    embedding_col = fc.embedding_column(
+        fc.sparse_column_with_hash_bucket(
+            "sparse_column_for_embedding", hash_bucket_size=10),
+        dimension=4)
+    real_valued_col1 = fc.real_valued_column("real_valued_column1")
+    bucketized_col1 = fc.bucketized_column(
+        fc.real_valued_column("real_valued_column_for_bucketization1"), [0, 4])
+    feature_columns = [sparse_col, embedding_col, real_valued_col1,
+                       bucketized_col1]
+
+    export_strategy = saved_model_export_utils.make_parsing_export_strategy(
+        feature_columns=feature_columns)
+    self.assertTrue(
+        isinstance(export_strategy, export_strategy_lib.ExportStrategy))
+
 
 def _create_test_export_dir(export_dir_base):
   export_dir = saved_model_export_utils.get_timestamped_export_dir(
       export_dir_base)
   gfile.MkDir(export_dir)
-  time.sleep(0.001)
+  time.sleep(2)
   return export_dir
 
 
