@@ -21,6 +21,7 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+import collections
 import hashlib
 import inspect
 import re
@@ -32,6 +33,7 @@ from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import op_def_registry
 from tensorflow.python.framework import ops
 from tensorflow.python.ops import array_ops
+from tensorflow.python.ops import resource_variable_ops
 from tensorflow.python.ops import variable_scope as vs
 from tensorflow.python.util import compat
 
@@ -66,12 +68,7 @@ def _get_node_def(op):
 
 
 def _get_op_def(op):
-  # pylint: disable=protected-access
-  if hasattr(op, "_sig"):
-    return getattr(op, "_sig")
-  else:
-    return op_def_registry.get_registered_ops()[op.type]
-  # pylint: enable=protected-access
+  return op.op_def or op_def_registry.get_registered_ops()[op.type]
 
 
 def _is_in_placeholders(op, func_arg_placeholders):
@@ -182,7 +179,7 @@ def _graph_to_function_def(graph, inputs, outputs, out_names=None):
   return func
 
 
-def _parse_kwargs_as_attrs(**kwargs):
+def _parse_kwargs_as_attrs(func_name, **kwargs):
   """Parses **kwargs into a node's attributes."""
   attrs = {}
 
@@ -191,8 +188,13 @@ def _parse_kwargs_as_attrs(**kwargs):
     attrs["_noinline"] = attr_value_pb2.AttrValue(b=bool(noinline))
 
   compiled = kwargs.pop("compiled", None)
+  separate_compiled_gradients = kwargs.pop("separate_compiled_gradients", None)
   if compiled is not None:
     attrs["_XlaCompile"] = attr_value_pb2.AttrValue(b=bool(compiled))
+    attrs["_XlaSeparateCompiledGradients"] = attr_value_pb2.AttrValue(
+        b=bool(separate_compiled_gradients))
+    attrs["_XlaScope"] = attr_value_pb2.AttrValue(
+        s=("function_%s" % func_name).encode())
 
   if kwargs:
     raise ValueError("Unknown keyword arguments: %s" % kwargs.keys())
@@ -222,9 +224,9 @@ def _call(sig, *inputs, **kwargs):
         'noinline'.
 
   Returns:
-     A Tensor if the function returns a single value; a list of Tensors
-     if the functio returns multiple value; the Operation if the function
-     returns no values.
+     A 2-element tuple. First element: a Tensor if the function returns a single
+     value; a list of Tensors if the function returns multiple value; the
+     Operation if the function returns no values. Second element: the Operation.
 
   Raises:
     ValueError: if the arguments are invalid.
@@ -233,9 +235,9 @@ def _call(sig, *inputs, **kwargs):
     raise ValueError("Expected number of arguments: %d, received: %d" %
                      (len(sig.input_arg), len(inputs)))
   name = kwargs.pop("name", None)
-  attrs = _parse_kwargs_as_attrs(**kwargs)
   g = ops.get_default_graph()
   func_name = sig.name
+  attrs = _parse_kwargs_as_attrs(func_name, **kwargs)
   output_types = [dtypes.DType(x.type) for x in sig.output_arg]
   with ops.name_scope(name, func_name, inputs) as name:
     op = g.create_op(
@@ -244,15 +246,16 @@ def _call(sig, *inputs, **kwargs):
         output_types,
         name=name,
         attrs=attrs,
+        op_def=sig,
         compute_shapes=False)
-  setattr(op, "_sig", sig)  # Remember the signature.
   if op.outputs:
     if len(op.outputs) == 1:
-      return op.outputs[0]
+      ret = op.outputs[0]
     else:
-      return tuple(op.outputs)
+      ret = tuple(op.outputs)
   else:
-    return op
+    ret = op
+  return ret, op
 
 
 def _get_func_name(func):
@@ -297,7 +300,8 @@ class _FuncGraph(ops.Graph):
              dtype=None,
              initializer=None,
              trainable=True,
-             collections=None,
+             collections=None,  # pylint: disable=redefined-outer-name
+             use_resource=None,
              **kwargs):
     """A custom variable getter."""
     # Here, we switch the default graph to the outer graph and ask the
@@ -316,8 +320,15 @@ class _FuncGraph(ops.Graph):
           dtype=dtype,
           initializer=initializer,
           trainable=trainable,
-          collections=collections)
+          collections=collections,
+          use_resource=use_resource)
       self.extra_vars.append(var)
+      if isinstance(var, resource_variable_ops.ResourceVariable):
+        # For resource-based variables read the variable outside the function
+        # and pass in the value. This ensures that the function is pure and
+        # differentiable. TODO(apassos) this may have performance problems if
+        # the function will only do embedding lookups on the variable.
+        return var.value()
       return var
 
   def create_op(self, op_type, inputs, data_types, **kwargs):
@@ -404,6 +415,7 @@ class _DefinedFunction(object):
                grad_func=None,
                python_grad_func=None,
                out_names=None,
+               shape_func=None,
                **kwargs):
     """Creates _DefinedFunction.
 
@@ -420,6 +432,8 @@ class _DefinedFunction(object):
         the function python-side.
       out_names: An optional list of strings for the function return value
         names.
+      shape_func: An optional function mapping an op to a list of static
+        output shapes.
       **kwargs: The keyword arguments. **kwargs is passed to every call
         site of this function.
 
@@ -433,8 +447,10 @@ class _DefinedFunction(object):
     self._grad_func = grad_func
     self._python_grad_func = python_grad_func
     self._out_names = out_names
+    self._shape_func = shape_func
     self._extra_kwargs = kwargs
     self._definition = None  # Constructed lazily.
+    self._sub_functions = dict()  # Constructed with definition.
 
     self._args = []
     assert isinstance(input_types, (list, tuple))
@@ -479,6 +495,7 @@ class _DefinedFunction(object):
   @property
   def captured_inputs(self):
     """Returns the list of implicitly captured inputs."""
+    self._create_definition_if_needed()
     return self._extra_inputs
 
   def _create_definition_if_needed(self):
@@ -507,72 +524,80 @@ class _DefinedFunction(object):
       outputs = [ops.convert_to_tensor(_) for _ in outputs]
     self._extra_inputs = temp_graph.extra_inputs
     inputs.extend(temp_graph.extra_args)
+    # pylint: disable=protected-access
+    self._sub_functions = temp_graph._functions
+    # pylint: enable=protected-access
 
     # Build the FunctionDef
     self._definition = _graph_to_function_def(
         temp_graph, inputs, outputs, out_names=self._out_names)
 
     # Extra kwargs are treated as attrs on the function def.
-    kwargs_attr = _parse_kwargs_as_attrs(**self._extra_kwargs)
+    sig_pre_func_name = self._func_name or _get_func_name(self._func)
+    kwargs_attr = _parse_kwargs_as_attrs(
+        sig_pre_func_name, **self._extra_kwargs)
     for k in kwargs_attr:
       self._definition.attr[k].CopyFrom(kwargs_attr[k])
 
     # Hash the definition and its dependencies.
-    hasher = hashlib.sha1()
-
-    def _hash_func_def():
-      """Hash the function definition agnostic to node/map ordering."""
-
-      def update_num(n):
-        hasher.update(compat.as_bytes("%x" % n))
-
-      def update_str(s):
-        update_num(len(s))
-        hasher.update(compat.as_bytes(s))
-
-      def update_strs(slist):
-        update_num(len(slist))
-        for s in slist:
-          update_str(s)
-
-      for adef in self._definition.signature.input_arg:
-        update_str(adef.SerializeToString())
-
-      for adef in self._definition.signature.output_arg:
-        update_str(adef.SerializeToString())
-
-      for n in sorted(self._definition.node_def, key=lambda n: n.name):
-        update_str(n.name)
-        update_str(n.op)
-        update_strs(n.input)
-        update_num(len(n.attr))
-        # NOTE: protobuf map serialization does not guarantee ordering.
-        for k in sorted(n.attr):
-          update_str(k)
-          update_str(n.attr[k].SerializeToString())
-
-    _hash_func_def()
-    # pylint: disable=protected-access
-    self._sub_functions = temp_graph._functions
-    for subname in sorted(self._sub_functions.keys()):
-      hasher.update(compat.as_bytes(self._sub_functions[subname]._hash_str))
-    # pylint: enable=protected-access
-
-    # Uses the first 8 bytes sha1 hash digest as the __hash__.
-    self._hash_str = hasher.hexdigest()[:8]
-    self._hash = int(self._hash_str, 16)
+    self._hash_str = self._create_hash_str(
+        self._definition.signature.input_arg,
+        self._definition.signature.output_arg,
+        self._definition.node_def)
 
     # Finally, we decide the function name to use.  If not specified,
-    # make up something which is almost certainly unique.
+    # make up something which is almost certainly unique (but deterministic).
     if not self._func_name:
       self._func_name = "_".join([_get_func_name(self._func), self._hash_str])
     self._definition.signature.name = self._func_name
     if self._func.__doc__:
       self._definition.signature.description = self._func.__doc__
 
-  def __hash__(self):
-    self._create_definition_if_needed()
-    return self._hash
+  def _create_hash_str(self, input_arg, output_arg, node_def):
+    """Creates an 8-character string unique to this input.
+
+    Args:
+      input_arg: the input_arg field of an OpDef
+                 (e.g. self._definition.signature.input_arg)
+      output_arg: the output_arg field of an OpDef
+                 (e.g. self._definition.signature.output_arg)
+      node_def: the node_def field of a FunctionDef
+                (e.g. self._definition.node_def)
+
+    Returns:
+      The unique string for this input
+    """
+    hasher = hashlib.sha1()
+
+    def update_num(n):
+      hasher.update(compat.as_bytes("%x" % n))
+
+    def update_str(s):
+      update_num(len(s))
+      hasher.update(compat.as_bytes(s))
+
+    def update_strs(slist):
+      update_num(len(slist))
+      for s in slist:
+        update_str(s)
+
+    for adef in input_arg:
+      update_str(adef.SerializeToString())
+
+    for adef in output_arg:
+      update_str(adef.SerializeToString())
+
+    for n in sorted(node_def, key=lambda n: n.name):
+      update_str(n.name)
+      update_str(n.op)
+      update_strs(n.input)
+      update_num(len(n.attr))
+      # NOTE: protobuf map serialization does not guarantee ordering.
+      for k in sorted(n.attr):
+        update_str(k)
+        update_str(n.attr[k].SerializeToString())
+
+    return hasher.hexdigest()[:8]
 
   def add_to_graph(self, g):
     """Adds this function into the graph g."""
@@ -581,7 +606,7 @@ class _DefinedFunction(object):
     # pylint: disable=protected-access
     # If 'g' has an identical function already, do nothing.
     prev = g._get_function(self.name)
-    if prev and (prev._hash == self._hash):
+    if prev and (prev._hash_str == self._hash_str):
       return
 
     # Adds this function into 'g'.
@@ -599,7 +624,115 @@ class _DefinedFunction(object):
   def __call__(self, *args, **kwargs):
     self.add_to_graph(ops.get_default_graph())
     args = [ops.convert_to_tensor(_) for _ in args] + self._extra_inputs
-    return _call(self._definition.signature, *args, **kwargs)
+    ret, op = _call(self._definition.signature, *args, **kwargs)
+    if self._shape_func is not None:
+      shapes = self._shape_func(op)
+      if len(shapes) != len(op.outputs):
+        raise ValueError("shape_func produced %d shapes for %d outputs" %
+                         (len(shapes), len(op.outputs)))
+      for (t, shape) in zip(op.outputs, shapes):
+        t.set_shape(shape)
+    return ret
+
+
+def _from_definition(fdef, grad_func=None):
+  """Creates a _DefinedFunction initialized from a FunctionDef proto.
+
+  Args:
+    fdef: a FunctionDef
+    grad_func: a _DefinedFunction or None
+
+  Returns:
+    A _DefinedFunction representing fdef
+  """
+  # The Python callable is only needed to create a FunctionDef. Since we have
+  # the FunctionDef here, we don't need to set _DefinedFunction._func (nor do we
+  # have access to such a callable here).
+  func = None
+  argnames = [arg.name for arg in fdef.signature.input_arg]
+  input_types = tuple(dtypes.as_dtype(arg.type)
+                      for arg in fdef.signature.input_arg)
+  func_name = fdef.signature.name
+  # Note: FunctionDefs do not include python gradient functions, so if the
+  # original _DefinedFunction included one it will not be reflected here.
+  python_grad_func = None
+  out_names = [arg.name for arg in fdef.signature.output_arg]
+  result = _DefinedFunction(func, argnames, input_types, func_name, grad_func,
+                            python_grad_func, out_names)
+  # pylint: disable=protected-access
+  result._definition = fdef
+  # Captured inputs are added as regular inputs to a function when it's
+  # serialized, i.e. any extra inputs from the original function are now
+  # included in `result`._args
+  result._extra_inputs = []
+  result._hash_str = result._create_hash_str(
+      result._definition.signature.input_arg,
+      result._definition.signature.output_arg,
+      result._definition.node_def)
+  # pylint: enable=protected-access
+  return result
+
+
+def _from_library(lib):
+  """Creates _DefinedFunctions initialized from a FunctionDefLibrary proto.
+
+  This method handles assigning the correct gradient functions to each
+  function.
+
+  Args:
+    lib: a FunctionDefLibrary
+
+  Returns:
+    A list of _DefinedFunctions
+
+  Raises:
+    ValueError: `lib` is invalid
+  """
+  if not lib.function and not lib.gradient: return []
+
+  # function name -> FunctionDef proto
+  funcs = {fdef.signature.name: fdef for fdef in lib.function}
+
+  # Validate that all references function names have function defs
+  for g in lib.gradient:
+    if g.function_name not in funcs:
+      raise ValueError("FunctionDefLibrary missing '%s' FunctionDef\n%s" %
+                       (g.function_name, str(lib)))
+    if g.gradient_func not in funcs:
+      raise ValueError("FunctionDefLibrary missing '%s' FunctionDef\n%s" %
+                       (g.gradient_func, str(lib)))
+
+  # function name -> gradient function name
+  func_to_grad = collections.defaultdict(lambda: None)
+  # gradient function name -> names of functions having that grad function
+  grad_to_funcs = collections.defaultdict(list)
+
+  for gdef in lib.gradient:
+    func_to_grad[gdef.function_name] = gdef.gradient_func
+    grad_to_funcs[gdef.gradient_func].append(gdef.function_name)
+
+  # Start with functions without gradients
+  ready = [fdef for fdef in lib.function
+           if func_to_grad[fdef.signature.name] is None]
+  if not ready:
+    raise ValueError("FunctionDefLibrary contains cyclic gradient functions!\n"
+                     + str(lib))
+  # function name -> _DefinedFunction
+  initialized = {}
+
+  while ready:
+    fdef = ready.pop()
+    name = fdef.signature.name
+
+    grad = initialized.get(func_to_grad[name])
+    if func_to_grad[name]: assert grad
+    defined_func = _from_definition(fdef, grad_func=grad)
+    initialized[name] = defined_func
+
+    ready.extend(funcs[f] for f in grad_to_funcs[name])
+
+  return initialized.values()
+
 
 # NOTE: The list needs to be extended when more data types are added.
 _DTYPE_TO_STR = {
@@ -749,6 +882,10 @@ class Defun(object):
   default graph. Because the addition of the function into the graph
   is deferred, the decorator can be used anywhere in the program.
 
+  Definitions of functions are frozen in a graph as soon as the graph is used to
+  create a session. Therefore, nodes using the function must be created in the
+  graph before the corresponding session is created.
+
   Example, but also see the [How To on functions](link_needed).
 
   ```python
@@ -762,9 +899,6 @@ class Defun(object):
   b = tf.Constant([2.0])
   c, d = MyFunc(a, b, name='mycall')
   ```
-
-  @@__init__
-
   """
 
   def __init__(self, *input_types, **kwargs):
@@ -793,6 +927,9 @@ class Defun(object):
 
          out_names = (optional). A list of strings, one per output
            tensor.
+
+         shape_func - (optional). A function taking the op and returning a list
+           of static shapes to set for the function's outputs.
     """
     self._input_types = input_types
     self._func_name = kwargs.pop("func_name", None)
@@ -857,7 +994,7 @@ class Declare(object):
 
   For example,
     # Declares  a function Foo, which takes a tf.int32 named "n" and a
-    # tf.float32 named "n" as inputs and returns a tf.float32 named "z"
+    # tf.float32 named "x" as inputs and returns a tf.float32 named "z"
     # as its output.
     foo = Declare("Foo", [("n", tf.int32), ("x", tf.float32)],
                   [("z", tf.float32)])
@@ -897,4 +1034,4 @@ class Declare(object):
 
   def __call__(self, *inputs, **kwargs):
     inputs = [ops.convert_to_tensor(_) for _ in inputs]
-    return _call(self._sig, *inputs, **kwargs)
+    return _call(self._sig, *inputs, **kwargs)[0]
